package com.fossgalaxy.games.tbs.metrics;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.order.Order;

/**
 * Stuff to keep track of every turn.
 *
 */
public class TurnMetrics {
	public static boolean DETAILED_LOGGING = true;
	
	private int turnNumber;
	private int playerID;
	
	private Map<UUID, Order> orders;
	private List<UUID> idle;
	private List<UUID> automatic;
	
	private Integer winner;
	private Set<Integer> losers;
	
	private GameState start;
	private GameState end;
	
	public TurnMetrics(int turnNumber, int playerID) {
		this.turnNumber = turnNumber;
		this.playerID = playerID;
		
		this.orders = new LinkedHashMap<>();
		this.idle = new ArrayList<>();
		this.automatic = new ArrayList<>();
	}
	
	public void logStateStart(GameState state) {
		if (DETAILED_LOGGING) {
			this.start = new GameState(state);
		}
	}
	
	public void logOrder(UUID id, Order order) {
		orders.put(id, order);
	}
	
	public void logIdle(UUID id) {
		idle.add(id);
	}

	public void logAutomatic(UUID id) {
		automatic.add(id);
	}

	public void logWinners(Integer winner, Set<Integer> losers) {
		this.winner = winner;
		this.losers = losers;
	}

	public void logStateEnd(GameState state) {
		if (DETAILED_LOGGING) {
			this.end = new GameState(state);
		}
	}
	
	public int getPlayerID() {
		return playerID;
	}
	
	public Integer getWinner() {
		return winner;
	}
	
	public Set<Integer> getLosers(){
		return losers;
	}
	
	public List<UUID> getIdleEntities() {
		return idle;
	}
	
	public Map<UUID, Order> getOrders() {
		return orders;
	}
	
	/**
	 * The turn number the game engine reported
	 * 
	 * @return the turn number, as reported by the engine
	 */
	public int getTurnNumber() {
		return turnNumber;
	}
	
	/**
	 * The list of entities the game engine made the moves for.
	 * 
	 * You can query the order map for what orders the game engine issued.
	 * 
	 * @return the UUIDs of the entities that made automatic actions.
	 */
	public List<UUID> getAutomaticEntities() {
		return automatic;
	}
	
	/**
	 * The game state before the player makes moves.
	 * 
	 * If DETAILED_LOGGING is turned off, this will return null.
	 * 
	 * @return the game state at the start of the turn
	 */
	public GameState getStateAtStart() {
		return start;
	}
	
	/**
	 * The game state after the player has made all their moves.
	 * 
	 * If DETAILED_LOGGING is turned off, this will return null.
	 * 
	 * @return the game state at the end of the turn
	 */
	public GameState getStateAtEnd() {
		return end;
	}
	
}
